macro "Travelogue"
    // This add-in lets you click on a map feature and view or print a 
    // set of pictures for the feature.

    // When run, it displays a toolbox with an View Tool and a Print Tool.

    // When you press the View Tool and click on a feature, pictures associated
    // with the feature (using an index file) are displayed in alphabetical 
    // order based upon the picture file name.  You click the mouse to move 
    // on to the next picture.

    // When you press the Printer Tool and click on a map feature, a 
    // layout is opened using the default printer.  Two pictures are added
    // to the layout using their file names as titles.  A page number is 
    // is added, and the layout is automatically printed.
    // This is repeated until all pictures are printed, then a message is
    // displayed telling the user how many pages were printed.

    // A specially-prepared dBASE file (the "Index file") is used to
    // obtain a list of the pictures to display or print.  It contains one
    // record for each combination of map feature name and picture file name.
    // It contains a field for the name of the feature (feat_name) and a
    // a field for the file name and extension of the picture (pict_name).  
    // All pictures are assumed to be in one directory which is specified
    // in the Administrative Settings section below in the pictures_dir variable.
    // This directory is prefixed to each pict_name to provide the full
    // path to the picture file.

    // Normally you will create the index file yourself and specify its location
    // in the Administrative Settings section below.  For demonstration purposes 
    // this add-in creates an index file for the .bmp files in the Maptitude 
    // tutorial folder For France and Italy.

    // You can adapt this add-in to use your own pictures and the geography 
    // corrsponding to the locations of your pictures.

    // Note: Standard Maptitude uses "Link Fields" to contain the names of slides
    // or slide shows to be displayed for a feature.  Link fields are fields in  
    // the attribute table of the layer with "File" in the name.  They contain 
    // the full path and file name of an image to be displayed, or they can 
    // contain the name of a slide show file, with extension of .sli, which 
    // contains a list of images to display.  This add-in show an alternative 
    // to this method when it makes sense to keep your image names in a database
    // without the full path.

    // This add-in demonstrates:
    // 1. Creating a toolbox with Tool items.
    // 2. Creating and populating a table.
    // 3. Creating and playing a slide show.
    // 4. Creating a layout, adding annotations to it and printing it.
    // 5. Creating a selection set and looping through the set.

    // To run this macro in test mode using the Maptitude GISDK toolbox:
    // 1. Choose File-Open to open europe.map in the Tutorial folder of 
    //    your Maptitude program folder.
    // 2. Make sure your default printer is set to Portrait orientation.  To 
    //    check it, choose Start-Settings-Printers on the WIndows taskbar. 
    //    Right-click on the printer you want to use and choose Set as Default
    //    Printer. Right click again, choose Properties and set the orientation 
    //    to Portrait.
    // 3. Choose Tools-Add-Ins and open the GIS Developer's Kit toolbox.
    // 4. Compile this macro in test mode using the first button (Compile).
    // 5. Run this macro by clicking the second button (Test), typing the macro 
    //    name "Travelogue", and clicking OK. The type of add-in is macro
    //    (the default).  This will display a toolbox in the upper left hand
    //    corner of the screen with two tool icons--View Pictures and Print
    //    Pictures. Hover the mouse over each icon to see the help text.
    // 6. Click the left tool, then click on Italy.  A picture is displayed.
    //    Click the mouse on the picture to move on to the next picture.
    // 7. Click the right tool, then click on Italy.  A layout is opened, two 
    //    pictures are added to it, along with picture titles and a page number
    //    and the layout is printed.  This is repeated until all pictures (four
    //    for Italy) are printed.
    // 8. Try steps 6 and 7 again by clicking on France.  These are only two 
    //    countries for which there are pictures in the Tutorial folder.

    // Administrative Settings --------------------------------------------------
    // This section contains variables that won't change while the macro is in
    // use, but make it easy to adapt the macro to a new situation. You could
    // also let the user set or change these variables in a dialog box.

    // Share the following variables for use by several macros.
    shared pictures_dir, features_layer, feature_name_field, index_view

    // Directory containing the photos to be displayed.
    pictures_dir="tutorial\\"  // Relative to the Maptitude program folder

    // Layer on which user clicks features to display associated images.
    features_layer="Country"  // Layer name of the tutorial\mtworld.cdf 
                              // database in the tutorial\europe.map

    // Feature name field used to look up images in the images database.
    feature_name_field="name"

    // Name of pictures index dbase file containing the feature name and the
    // picture file name, one record for each picture displayed for a feature.
    index_filename="Tutorial\\Travelogue picture index.dbf"

    // Create demonstration picture index file.  Normally you will create and 
    // maintain this file outside of this add-in.  Note: the feature name
    // (feat_name) is indexed because we will be selecting based on the field.
    // If the index file view is open then close it.
    on notfound goto skip_close_index
    CloseView("index_table")
    skip_close_index:
    on notfound default

    fields={
    	{"feat_name", "String", 25, null, "Yes"},  
    	{"pict_name", "String", 25, null, "No"}}
    index_view=CreateTable("index_table", index_filename, "dBase", fields)

    // Add the tutorial bitmap files.
    values={
        {"France", "PARIS.BMP"},
        {"Italy", "VENICE.BMP"},
        {"Italy", "MURANO.BMP"},
        {"Italy", "GONDOLA.BMP"},
        {"Italy", "FLORENCE.BMP"}}
    AddRecords(index_view , {"feat_name", "pict_name"},	values,	null)
    CloseView(index_view)
    // End Administrative Settings ----------------------------------------------

    // If map is not open then prompt user to open it.
    map=GetMap()
    if map = null then do
	   ShowMessage("Please open the europe.map in the Maptitude "+
            "install directory, Tutorial folder.")
       goto endit
       end

    // If the features layer is not in the map then prompt user to add it.
    layers = GetLayers()
    if ArrayPosition(layers[1], {features_layer},)=0 then do
        ShowMessage("Please add the tutorial\mtworld.cdf in your "+
            "Maptitude install directory to your map.")
        goto endit
        end

    // Close the toolbox if it is already open; if not found skip error.
    on notfound goto skip_close
    CloseDbox("pictures_toolbox")
    skip_close:
    on notfound default

    // Open the pictures index file.
    index_view = OpenTable("index_table", "dBase", {index_filename,})

    // Run the toolbox.
    RunDbox("pictures_toolbox", index_view)
    endit:
endmacro

// A toolbox with two tools in it.  Note the "nokeyboard" option.
// Without this option the user must click twice on the map; first to
// change the focus to the map, and second to select a feature.
// With nokeyboard, the user only needs to click once to select a feature.
dbox "pictures_toolbox" (index_view) 10,10 toolbox title: "Toolbox" nokeyboard
    Init Do
      shared pictures_dir, features_layer, feature_name_field, index_view
      hppi=7200 // Number of hundredths of points per inch; there are 72 points 
                // per inch. This is used below to make it easier to specify   
                // layout positions in inches.
    Enditem

    // Close Do allows user to close by clicking in upper right hand corner.
    Close Do
        // Close the index file view.
        // Include error trap in case user used File-Close All in Maptitude.
        on notfound goto skip_close  
        CloseView(index_view)
        skip_close:
        on notfound default
        Return()  
    EndItem

    Tool "display_tool" 0, 0 icons: "bmp\\buttons|237.bmp", "bmp\\buttons|271.bmp"
            Cursor: "Arrow" Help: "View Pictures"
        Do

            // Get a click from the user.
            pt=ClickCoord()

            // Find the map feature containing the point.
            SetLayer(features_layer)
            rh = LocateNearestRecord(pt, 0.0)

            // If no feature is found then display message and skip to end.
            if rh=null then ShowMessage("No feature selected.")
            else do
            
                // Call macro to get list of pictures for selected feature.
                // Note, the name of the field from which to get the feature name
                // is contained in the variable named "feature_name_field".
                // To construct a field spec with a variable you enclose it in
                // parentheses as shown in last parameter of this macro call.
                filenames=RunMacro("get_pictures",
                    features_layer.(feature_name_field))

                // If no pictures available then display message and go to end.
                if filenames=null then do
                    ShowMessage("No pictures available.")
                    goto end_display_tool
                    end

                // Build a slideshow array from the filenames.
                dim slides[filenames.length]
                for i=1 to filenames.length do
                    parts=SplitPath(filenames[i])
                    slides[i]={filenames[i],,parts[3],,}
                    end

                //  Play the slide show.
                PlaySlideShow(slides,)
                end

            end_display_tool:
        EndItem

    Tool "print_tool" after, same icons: "bmp\\buttons|124.bmp",
            "bmp\\buttons|158.bmp" Cursor: "Arrow" Help: "Print Pictures"
        Do
            // Get a click from the user.
            pt=ClickCoord()

            // Find the map feature containing the point.
            SetLayer(features_layer)
            rh = LocateNearestRecord(pt, 0.0)

            // If no feature found then display message and skip to end.
            if rh=null then ShowMessage("No feature selected.")
            else do
            
                // Call macro to get list of pictures for selected feature.
                filenames=RunMacro("get_pictures",
                    features_layer.(feature_name_field))

                // If no pictures available then display message and go to end
                if filenames=null then do
                    ShowMessage("No pictures available.")
                    goto end_print_tool
                    end

                // Print the pictures, two to a page, with file name as picture 
                // title below the picture.  Also print page number.

                // Open a layout using the default printer.
                CreateLayout("my layout", )
                layout = GetLayout()

                // If you want the user to be able to select the printer and set 
                // properties, then activate the next line.
                // LayoutProperties()

                num=1   // This counter indicates whether the 1st or 2nd picture  
                        // on the page is being added

                // Loop thru all pictures.
                num_to_print=filenames.length
                page_no=1
                for i=1 to num_to_print do

                // Set up location for upper picture on page using inches times 
                // hundredths of points per inch (hppi=7200).
                // Variable loc contains the coordinates of the upper left and 
                // lower right corner in which to display the picture.
                if num=1 then loc={Coord(r2i(2.7*hppi),r2i(0.3*hppi)), 
                    Coord(r2i(6.0*hppi),r2i(5.0*hppi))}

                // Set up location for lower picture on page.
                if num=2 then loc={Coord(r2i(2.7*hppi),r2i(5.7*hppi)), 
                    Coord(r2i(6.0*hppi),r2i(10*hppi))}

                // Use the file name with no path or extension as the title.
                parts=SplitPath(filenames[i])
                title=proper(parts[3])

                // Calculate position of the picture title centered just below 
                // the picture.  Note: 0,0 is in the upper left hand corner of 
                // the layout, so by adding to the lower picture "latitude" you 
                // move it down the page.
                text_loc={Coord(loc[1].lon, r2i(loc[2].lat+1000)), 
                    Coord(loc[2].lon, r2i(loc[2].lat+2000))}

                // If the first picture, then clear previous annotations.
                if num=1 then do
                    SelectAllAnnotations()
                    DropSelectedAnnotations()
                    end

                // Add the page number.
                if num=1 then do
                    AddAnnotation("Layout|"+layout,"Text",{
	                    {"Color",ColorRGB(0,0,0)},
	                    {"Font",'Arial|Bold|12'},
	                    {"Location",{coord(r2i(7.6*hppi), r2i(10.5*hppi)),
                            coord(r2i(8.2*hppi), r2i(10.6*hppi))}},
	                    {"Text",'Page '+i2s(page_no)}})
                    end
        
                // Add the picture.
                AddAnnotation("Layout|"+layout, "Bitmap", {
                	{"File", filenames[i]},
                	{"Location", loc}})

                // Add the title, just under, left aligned with left side
                // of the picture.
                AddAnnotation("Layout|"+layout,"Text",{
                	{"Color",ColorRGB(0,0,0)},
                	{"Font",'Arial|Bold|16'},
                	{"Location",text_loc},
                	{"Text",title}})

                // If the second picture then print and increment page number.
                if num=2 then do
                    PrintLayout(1,1)
                    page_no=page_no+1  // Increment the page number
                    end
                num=num+1
                if num=3 then num=1
                end  // End of pictures loop

                // If an odd number of pictures were printed then last
                // page only has one on it, so print it now.
                if mod(num_to_print,2)<>0 then do
                    Printlayout(1,1)
                    page_no=page_no+1  // Increment the page number
                    end

                // Close the layout.
                CloseLayout(layout)

                // Let user know printouts were done, because it can happen
                // very fast.
                num = page_no - 1
                if num=1 then ShowMessage('1 page printed')
                else ShowMessage(i2s(num)+' pages printed')
                end
            end_print_tool:
        EndItem
EndDbox 

Macro 'get_pictures' (feature_name)
    // Uses feature_name to select pictures in the index file (index_view).
    // Prefixes each picture file name with the pictures directory (pictures_dir),
    // Alphabetizes the pictures by name and returns the list of full picture
    // file names in an array.  If no pictures are found, then null is returned.
    
    shared pictures_dir, features_layer, feature_name_field, index_view
    setview(index_view)

    // Select the index records that have feat_name matching the passed in name.
    // Note: the \" is necessary to insert an quote in a string.  The \ is called
    // the "escape" character. Not needed if you use single quotes around a string.
    qry = "Select * where feat_name = \""+feature_name+"\""
    n=SelectByQuery("Selection", "Several", qry,)

    // If no pictures are found, return null.
    if n=0 then return(null)
    else do

        // Loop through the "Selection" selection set and build an array of the 
        // full path and file names for all of the pictures.
        results=null
        rh=GetFirstRecord(index_view+"|Selection",)
        while rh<>null do
            // This statement shows how you can build an array by adding elements
            // one at a time.
            results=results+{pictures_dir+index_view.pict_name}
            rh=GetNextRecord(index_view+"|Selection",,)
        end

        // Sort the array.
        results=SortArray(results)

        // Return the results as an array.
        return(results)
        end
EndMacro
              

